package org.jutils.collection;

import java.util.*;
/**
 * /*------------------------------------------------------------------------------
 * Name:      IntList
 * Project:   jutils.org
 * Copyright: jutils.org, see jutils-LICENSE file
 * Comment:   Contains a sorted array of int values.
 * Version:   $Id: IntList.java,v 1.3 2000/07/13 14:45:59 winkler Exp $
 * ------------------------------------------------------------------------------*-/
 * @author: M.Winkler, doubleSlash GmbH
 */
public class IntList extends Observable implements DynamicList {
   /** If no initial size is provided, this value is taken. */
   public static final int DEFAULT_INITIAL_SIZE = 50;

   /** The number of ints that are added to the internal array when more capacity is used. */
   private int raiseAmount_ = 0;

   /** The internal array is layed out as a ring buffer; so this is the index of the first element. */
   private int firstIndex_ = 0;

   /** This is the index of the last element. */
   private int lastIndex_ = -1;

   /** The number of elements in the internal array. */
   private int count_ = 0;

   /** The internal array where the concrete data is hold. Layed out as a ring buffer. */
   private int[] intList_ = null;

   /** Indicates that the next read access will initiate a sort. This is set true when adding an element. */
   private boolean sortRequired_ = false;

   /** The chars that are used to create a string representation. */
   private char[] listChars = {'[', ',', ']'};

   /**
   * An array that is used when sorting. For this array is used with multiple instances, this
   * one should be synchronized upon when used!
   **/
   private static int[] tempSortArray = null;
/**
 * IntList constructor comment.
 */
public IntList() {
   this (DEFAULT_INITIAL_SIZE);
}
/**
 * @author: M.Winkler, doubleSlash GmbH
 * @param initialSize int
 */
public IntList(int initialSize) {
   intList_ = new int[initialSize];
   firstIndex_ = 0;
   count_ = 0;
   raiseAmount_ = initialSize;
}
/**
 * @author: M.Winkler, doubleSlash GmbH
 * @return int
 */
public synchronized int capacity() {
   return intList_.length;
}
/**
 * @author: M.Winkler, doubleSlash GmbH
 * @return int
 * @param a int[]
 */
public synchronized int copyInto(int[] a) {

   // parameter check
   if ((a == null) || (a.length < count_))
      throw new ArrayStoreException("passed array has less elements than required.");

   // check consistency
   pReensureConsistency();

   // copy array while locking it.
   synchronized (intList_) {
      System.arraycopy(intList_, 0, a, 0, count_);
   }
   return count_;
}
/**
 * @author: M.Winkler, doubleSlash GmbH
 * @return int
 */
public synchronized int count() {
   return count_;
}
/**
 * @author: M.Winkler, doubleSlash GmbH
 * @return int
 * @param newCapacity int
 */
public synchronized int ensureCapacity(int newCapacity) {
   pEnsureCapacity(newCapacity);
   return intList_.length;
}
/**
 * @author: M.Winkler, doubleSlash GmbH
 */
private void pCheckTempArray() {
   synchronized (tempSortArray) {
      if ((tempSortArray == null) || (tempSortArray.length < count_)) {
         int newTempSize = Math.max(DEFAULT_INITIAL_SIZE * 2, count_ * 2);
         tempSortArray = new int[newTempSize];
      }
   }
}
/**
 * Defragment the internal array, i.e. that firstElement == 0 ;
 * @author: M.Winkler, doubleSlash GmbH
 */
private void pDefrag() {

   // synchronize on intlist_; if done nothing, releasing is fast;
   // if done something, synchronizing is REQUIRED, so its okay, i think.
   synchronized (intList_) {

      // is it shifted?!?
      if (firstIndex_ > 0) {
         // is it fragmented?!?
         if (firstIndex_ + count_ > intList_.length) {

            // determine size of those two fragments
            int firstFragmentCount = intList_.length - firstIndex_ + 1;
            int secondFragmentCount = count_ - firstFragmentCount;

            // check temp array
            pCheckTempArray();

            // syncing on static tempArray (could be used by other instances).
            synchronized (tempSortArray) {

               // temporarily saving second (lower indexed) fragment
               System.arraycopy(intList_, 0, tempSortArray, 0, secondFragmentCount);

               // copying higher indexed fragment to start from index 0
               System.arraycopy(intList_, firstIndex_, intList_, 0, firstFragmentCount);

               // appending temporarily stored fragment back to intList_
               System.arraycopy(tempSortArray, 0, intList_, firstFragmentCount, secondFragmentCount);

               // correcting attributes.
               firstIndex_ = 0;
               lastIndex_ = count_ - 1;
            } // of "synchronized (tempSortArray)"
         } // of "if (firstIndex_ + count_ > intList_.length)"
         else {
            // its only shifted, and not fragmented.
            System.arraycopy(intList_, firstIndex_, intList_, 0, count_);
            firstIndex_ = 0;
            lastIndex_ = count_ - 1;
         } // of "else"
      } // of "if firstIndex_ > 0"
   } // of "synchronized (intList_)"
}
/**
 * @author: M.Winkler, doubleSlash GmbH
 * @return int
 * @param newCapacity int
 */
private void pEnsureCapacity(int newCapacity) {
   synchronized (intList_) {
      if (newCapacity > intList_.length) {

         // sort data
         pReensureConsistency();

         // calculate new capacity
         int minNewCapacity = intList_.length + raiseAmount_;
         if (newCapacity < minNewCapacity)
            newCapacity = minNewCapacity;

         // get new memory
         int[] newData = new int[newCapacity];

         // copy contents
         System.arraycopy(intList_, 0, newData, 0, count_);
         intList_ = newData;
      }
   }
   return;
}
/**
 * @author: M.Winkler, doubleSlash GmbH
 * @return int
 */
public static int performSemanticTest() {
   System.err.println("IntList.performSemanticTest() not yet implemented.");
   return 1;
}
/**
 * @author: M.Winkler, doubleSlash GmbH
 * @return int
 */
public synchronized int popMax() {
   if (count_ < 1)
      throw new IllegalStateException("Sorry, no elements to pop().");

   // defrag and sort array, if required.
   pReensureConsistency();

   // determine maximum value
   int returnValue = intList_[lastIndex_];

   // reduce content
   lastIndex_--;
   count_--;

   // rollunder?!?
   if (lastIndex_ < 0)
      lastIndex_ = intList_.length - 1;








   //
   return count_;
}
/**
 * @author: M.Winkler, doubleSlash GmbH
 * @return int
 */
public synchronized int popMin() {
   if (count_ < 1)
      throw new IllegalStateException("Sorry, no elements to pop().");

   // defrag and sort array, if required.
   pReensureConsistency();

   // determine maximum value
   int returnValue = intList_[lastIndex_];

   // reduce content
   lastIndex_--;
   count_--;

   // rollunder?!?
   if (lastIndex_ < 0)
      lastIndex_ = intList_.length - 1;

   // return the number of elements in list AFTER OPERATION
   return count_;
}
/**
 * Raises the capacity for exactly one step.
 * @author: M.Winkler, doubleSlash GmbH
 */
private void pRaiseCapacity() {
   int newCapacity = intList_.length + raiseAmount_;
   pEnsureCapacity(newCapacity);
}
/**
 * Sorts the internal array so that the following constraints are met:<br>
 * <li>firstIndex = 0 ;</li>
 * <li>all elements are sorted, starting with the lowest, ending with the greatest.</li>
 * @author: M.Winkler, doubleSlash GmbH
 */
private void pReensureConsistency() {

   // has there changed something ?!?
   if (sortRequired_) {

      // reunite parts if present
      pDefrag();
      if (count_ > 1) {

         // sort using arrays
         synchronized (intList_) {
            Arrays.sort(intList_, 0, count_);
         }
      }

      // reset dirty flag
      sortRequired_ = false;
   }
}
/**
 * @author: M.Winkler, doubleSlash GmbH
 * @return int
 * @param value int
 */
public synchronized int push(int value) {
   if (count_+1 > intList_.length)
      pRaiseCapacity();

   // save highest value
   int highestValue = intList_[lastIndex_];

   // raise content managing attributes
   lastIndex_++;
   count_++;

   // rollover?!?
   if (lastIndex_ >= intList_.length)
      lastIndex_ = 0;

   // store element
   intList_[lastIndex_] = value;

   // Check wether a sort is required. Sort is required, when the last
   // value (which should be the largest value) is greater than the
   // inserted value, or if sorting is required from a previous addition.
   sortRequired_ = sortRequired_ || (value < highestValue);

   // return the new count of values.
   return count_;
}
/**
 * Ensures that only that much space as needed is used.
 * @author: M.Winkler, doubleSlash GmbH
 */
public synchronized void shrinkToFit() {

   // first, defrag and sort array
   pReensureConsistency();

   // will modify list, so synchronize on it
   synchronized (intList_) {
      // allocate memory
      int newCapacity = count_;
      if (newCapacity < raiseAmount_)
         newCapacity = raiseAmount_;
      int[] newData = new int[newCapacity];

      // if any entries present, copy them to new data
      if (count_ > 0)
         System.arraycopy(intList_, 0, newData, 0, count_);
      intList_ = newData;
   }
}
/**
 * @author: M.Winkler, doubleSlash GmbH
 * @return int[]
 */
public synchronized int[] toArray() {
   int[] returnValue = null;
   if (count_ > 0) {
      pReensureConsistency();
      returnValue = new int[count_];
      System.arraycopy(intList_, 0, returnValue, 0, count_);

   }
   return null;
}
/**
 * getIndexOutOfBoundsBehavior method comment.
 */
public int getIndexOutOfBoundsBehavior() {
   return THROW_EXCEPTION;
}
/**
 * getObjectAt method comment.
 */
public java.lang.Object getObjectAt(int index) {
   return null;
}
/**
 * isAutoResizing method comment.
 */
public boolean isAutoResizing() {
   return true;
}
/**
 * setListChars method comment.
 */
public void setListChars(char openBracket, char separator, char closeBracket) {}
/**
 * <br>Creation date: (12.07.2000 14:54:50)<br>
 * @author: M.Winkler, doubleSlash GmbH
 *
 * @return java.lang.String
 */
public String toString() {
   return toString(listChars[0], listChars[1], listChars[2]);
}
/**
 * toString method comment.
 */
public java.lang.String toString(char openBracket, char separator, char closeBracket) {
   StringBuffer returnValue = new StringBuffer(1000);
   returnValue.append(openBracket);
   if (intList_.length > 0)
      for (int index = 0; index < intList_.length; index++) {
         if (index > 0)
            returnValue.append(separator);
         returnValue.append(intList_[index]);
      }
   returnValue.append(closeBracket);
   return returnValue.toString();
}
}
