package org.jutils.collection;

/*------------------------------------------------------------------------------
Name:      Queue.java
Project:   jutils.org
Copyright: jutils.org, see jutils-LICENSE file
Comment:   Queue for client messages
Version:   $Id: Queue.java,v 1.2 2000/07/13 08:43:15 winkler Exp $
Author:    ruff@swand.lake.de
------------------------------------------------------------------------------*/
import org.jutils.JUtilsException;

import java.util.List;
import java.util.LinkedList;
import java.util.ListIterator;
import java.util.Collections;
import java.util.NoSuchElementException;


/**
 * This queue (FIFO) implementation may serve as your base class
 * or as a member variable. <br />
 * It is based on a linked list.
 * <p />
 * TODO: Allow persistence store e.g. via JDBC bridge into Orcale with some smart caching
 * @version $Revision: 1.2 $
 * @author $Author: winkler $
 */
public class Queue
{
   private String ME = "Queue";
   private String name = "";
   private final int MAX_ENTRIES;
   /** Default is false, and you get an Exception if queue is full */
   private boolean discardOldest = false;

   /**
   * The queue is implemented with a linked list.
   * I believe ArrayList is more expensive in this case
   */
   private LinkedList queueList = null;

   /**
   * Constructs an empty FIFO queue.
   * @param name  A nice name, for error reporting only
   * @param maxEtnries The maximum number of nodes for this queue
   */
   public Queue(String name, int maxEntries) {
      this.name = name;
      this.MAX_ENTRIES = maxEntries;
      init();
   }

   /**
   * Default is that an Exception is thrown if the queue is full.
   * Calling this method changes the behavior that if the queue is
   * filled, the supernumerary entries are falling out at the end and are lost
   */
   public void setModeToDiscardOldest() {
      this.discardOldest = true;
   }

   /**
   * Allocates a new LinkedList.
   */
   private void init() {
      // this.queueList = (List)Collections.synchronizedList(new LinkedList());
      this.queueList = new LinkedList(); // !!! thread save?
   }

   /**
   * Add a unit to the beginning of the queue.
   * <p />
   * @exception no more space
   */
   public final void push(Object obj) throws JUtilsException {
      synchronized (queueList) {
         if (queueList.size() >= MAX_ENTRIES) {
            if (discardOldest)
               pull(); // Discard oldest
            else {
               throw new JUtilsException(ME + ".MaxSize", "Maximun size=" + MAX_ENTRIES + " of queue '" + name + "' reached");
            }
         }
         queueList.addFirst(obj);
      }
   }

   /**
   * pull the next unit from the end of the queue.
   * <p />
   * @return The object
   */
   public final Object pull() {
      try {
         synchronized (queueList) {
            return queueList.removeLast();
         }
      }
      catch (NoSuchElementException e) {
         return null;
      }
   }

   /**
   * Returns a list-iterator of the elements in this list (in proper sequence), starting at the specified position in the list.
   * <p />
   * TODO: Discuss thread safety when using this list
   * @parameter index of first element to be returned from the list-iterator (by a call to next).
   * @return a ListIterator of the elements in this list (in proper sequence), starting at the specified position in the list.
   * @exception IndexOutOfBoundsException - if index is out of range (index < 0 || index > size()).
   */
   public final ListIterator queueIterator(int index) throws IndexOutOfBoundsException {
      return queueList.listIterator(index);
   }

   /**
   * Check if the queue is filled up.
   * <p />
   * @return true space for at least on more entry
   *         false quota exceeded
   */
   public final boolean isFull() throws JUtilsException {
      if (queueList.size() >= MAX_ENTRIES)
         return true;
      return false;
   }

   /**
   * How many objects are in the queue.
   * <p />
   * @return number of objects
   */
   public final int size() {
      return queueList.size();
   }

   /**
   * Only for testing
   *    java org.jutils.collection.Queue
   */
   public static void main(String args[]) throws Exception {
      int size = 3;
      Queue queue = new Queue("Test", size);
      queue.setModeToDiscardOldest();
      try {
         queue.push("Hello ");
         queue.push("world ");
         queue.push("my ");
         queue.push("dear ");
         queue.push("friend!");
      }
      catch (JUtilsException e) {
      }
      StringBuffer result = new StringBuffer("");
      while (queue.size() > 0) {
         result.append(queue.pull());
      }
      System.out.println("For a queue with max size=" + size + " the result is: " + result.toString());
   }
}
