package org.jutils.init;

/*------------------------------------------------------------------------------
Name:      Property.java
Project:   jutils.org
Copyright: jutils.org, see jutils-LICENSE file
Comment:   Handling property variables for your project
Version:   $Id: Property.java,v 1.16 2000/07/13 12:19:36 ruff Exp $
------------------------------------------------------------------------------*/
import org.jutils.JUtilsException;
import org.jutils.text.StringHelper;
import org.jutils.io.FileUtil;

import java.io.*;
import java.util.Properties;
import java.util.Enumeration;


/**
 * Handling properties for your project, e.g. your property file $HOME/cool.properties.
 * <p />
 * This class searches the given property file in typical directories (see findPath() method).
 * The properties can be overwritten by properties given as a String[], for example
 * from the command line (see addArgs2Props() method for an explanation).
 * <p />
 * All environment variable (e.g. ${java.io.tmpdir}="/tmp", ${file.separator}="/") can be used
 * in the properties file and are replaced on occurrence.<br />
 * Further, you can use any of your own variables from the properties file or from the
 * command line with curly braces: ${...}.<br />
 * Example:<br />
 * A user may specify the MOMS_NAME and use it later as a variable as follows:<br />
 * <pre>   java -DMOMS_NAME=Joan ...</pre><br />
 * or in the properties file:<br />
 * <pre>   MOMS_NAME=Joan</pre>
 * or on the command line:<br />
 * <pre>   java org.MyApp  -MOMS_NAME Joan</pre><br />
 * and now use it like<br />
 * <pre>   MOMS_RECIPY=Strawberry cake from ${MONS_NAME} is the best.</pre>
 * which results to<br />
 * <pre>   MOMS_RECIPY=Strawberry cake from Joan is the best.</pre>
 * The maximum replacement (nesting) depth is 50.
 * <p />
 * A user may specify the PROJECT_HOME environment variable, the property file will be
 * searched there as well.
 * <p />
 * NOTE: These are often helpful variables
 * <ul>
 *    <li>${user.dir} =   The current directory </i>
 *    <li>${user.home} =  The users home directory</i>
 * </ul>
 * user.dir and user.home are without a path separator at the end
 * <p />
 * The property file settings are the weakest, they are overwritten
 * by system property settings and finally by command line arguments.
 *
 * @author Marcel Ruff
 */
public class Property
{
   private final static String ME = "Property";

   public final static String separator = System.getProperty("file.separator");
   /** The users home directory */
   public final static String userHome = System.getProperty("user.home");
   /** current directory, can it change for each instance or is it final in the JVM? */
   public String currentPath = System.getProperty("user.dir");
   /** user supplied path in PROJECT_HOME */
   public String projectHomePath = System.getProperty("PROJECT_HOME");
   /** The java home ext directory, e.g. /opt/jdk1.2.2/jre/lib/ext */
   public final static String javaHomeExt = System.getProperty("java.ext.dirs");
   /** The java home directory, e.g. /opt/jdk1.2.2/jre/lib */
   public final static String javaHome = System.getProperty("java.home");
   /** The supplied file name, e.g. cool.properties */
   private String propertyFileName = null;
   /** Holding all found properties */
   private Properties jutilsProperties = null;
   private final Properties dummyProperties = new Properties();
   /** Scan system environment variables? */
   private boolean scanSystemProperties = true;
   /** Replace occurrences of ${key} with the value of key */
   private boolean replaceVariables = true;
   /** Was the option '--help' or '-help' or '-h' or '-?' specified? */
   private boolean wantsHelp = false;

   /**
   * Construct a property container from supplied property file and args array.
   * <p />
   * @param fileName The property file name, e.g. "project.properties"
   * @param scanSystemProperties Scan System.getProperties() as well, you can
   *        add variable to JVM like: java -DmyName=Joe ...
   * @param args A String array with properties, usually from command line, e.g.
   *        java myApp -trace true -name Joe
   * @param replaceVariables true: replace occurrences of ${key} with the value of key
   */
   public Property(String fileName, boolean scanSystemProperties, String[] args, boolean replaceVariables) throws JUtilsException {
      System.out.println("Property: filenName=" + fileName + ", scanSystemProperties=" + scanSystemProperties + ", args=" + args + ", replaceVariables=" + replaceVariables);
      // System.out.println(org.jutils.log.Log.getStackTrace());
      this.propertyFileName = fileName;
      this.scanSystemProperties = scanSystemProperties;
      this.replaceVariables = replaceVariables;
      loadProps(args);
      if (jutilsProperties == null)
         jutilsProperties = dummyProperties;
   }

   /**
   * Get the internal handle.
   * @return The Properties handle.
   */
   public final java.util.Properties getProperties() {
      return jutilsProperties;
   }

   /**
   * Try to find the given key.
   * @param key the key to look for
   * @param defaultVal the default value to return if key is not found
   * @return The String value for the given key
   */
   public final String get(String key, String defaultVal) {
      String str = (String) jutilsProperties.get(key);
      if (str == null)
         return defaultVal;
      return str;
   }

   /**
   * Try to find the given key.
   * @param key the key to look for
   * @param defaultVal the default value to return if key is not found
   * @return The int value for the given key
   */
   public final int get(String key, int defaultVal) {
      String str = (String) jutilsProperties.get(key);
      if (str == null)
         return defaultVal;
      try {
         return Integer.parseInt(str);
      }
      catch (Exception e) {
         return defaultVal;
      }
   }

   /**
   * Try to find the given key.
   * @param key the key to look for
   * @param defaultVal the default value to return if key is not found
   * @return The long value for the given key
   */
   public final long get(String key, long defaultVal) {
      String str = (String) jutilsProperties.get(key);
      if (str == null)
         return defaultVal;
      try {
         return Long.parseLong(str);
      }
      catch (Exception e) {
         return defaultVal;
      }
   }

   /**
   * Try to find the given key.
   * <p />
   * See toBool() for a list of recognized strings.
   * @param key the key to look for
   * @param defaultVal the default value to return if key is not found
   * @return The boolean value for the given key
   */
   public final boolean get(String key, boolean defaultVal) {
      String str = (String) jutilsProperties.get(key);
      if (str == null)
         return defaultVal;
      try {
         return toBool(str);
      }
      catch (Exception e) {
         return defaultVal;
      }
   }

   /**
   * Try to find the given key.
   * <p />
   * @param key the parameter key to look for
   * @return true if the property exists
   */
   public final boolean propertyExists(String key) {
      String str = (String) jutilsProperties.get(key);
      if (str == null)
         return false;
      return true;
   }

   /**
   * Remove the given property.
   * <p />
   * This method does nothing if the key is not in the property hashtable.
   * @param key the key to remove
   */
   public final Object removeProperty(String key) {
      return jutilsProperties.remove(key);
   }

   /**
   * If set to true user wants you to display a usage text.
   * @return true if the option '--help' or '-help' or '-h' or '-?' was given.
   */
   public final boolean wantsHelp() {
      return wantsHelp;
   }

   /**
   * Use this method only the first time to initialize everything.
   * @args This key/value parameter array is added to the poperties object (see addArgs2Props()).
   * @return the initialized Properties
   */
   public final Properties loadProps(String[] args) throws JUtilsException {
      if (jutilsProperties != null) {
         System.out.println("Property: Reloading Property file ... args=" + args);
      }

      // set default
      jutilsProperties = new Properties();
      try {
         // 1. Read user supplied properties file
         if (propertyFileName != null) {
            String fileName = findFile(propertyFileName, args);
            if (fileName != null) {
               File file = new File(fileName);
               FileInputStream fis = new FileInputStream(file);
               jutilsProperties.load(fis);
               fis.close();
               System.out.println("Property: Loaded " + fileName);
            }
            else {
               System.err.println("Property: Please copy " + propertyFileName + " to your home directory. We continue with default settings.");
            }
         }
         else
            System.out.println("Property: No property file given.");

         // 2. Read system environment, e.g. java -Dname=joe
         if (scanSystemProperties == true) {
            Properties env = System.getProperties();
            for (Enumeration e = env.propertyNames(); e.hasMoreElements();) {
               String key = (String) e.nextElement();
               jutilsProperties.put(key, System.getProperty(key));
            }
         }
         else
            System.out.println("Property: No system properties scanned.");

         // 3. read user supplied String[], e.g. command line parameters
         if (args != null) {
            addArgs2Props(jutilsProperties, args);
         }
         else
            System.out.println("Property: No args array given.");


         // 4. Substitute dynamic variables, e.g. ${user.dir}
         if (replaceVariables == true)
            replaceVariables();
      }
      catch (IOException e) {
         throw new JUtilsException(ME + ".Error", "Unable to initialize " + propertyFileName + ": " + e);
      }
      return jutilsProperties;
   }

   /**
   * Replace dynamic variables, e.g. ${XY} with their values.
   * <p />
   * The maximum replacement (nesting) depth is 50.
   */
   private final void replaceVariables() throws JUtilsException {
      if (replaceVariables == false)
         return;
      for (Enumeration e = jutilsProperties.keys(); e.hasMoreElements();) {
         String key = (String) e.nextElement();
         String value = (String) jutilsProperties.get(key);
         for (int ii = 0;; ii++) {
            int from = value.indexOf("${");
            if (from != -1) {
               int to = value.indexOf("}", from);
               if (to == -1) {
                  throw new JUtilsException(ME + ".InvalidVariable", "Invalid variable '" + value.substring(from) + "', expecting ${} syntax.");
               }
               String sub = value.substring(from, to + 1); // "${XY}"
               String subKey = sub.substring(2, sub.length() - 1); // "XY"
               String subValue = (String) jutilsProperties.get(subKey);
               if (subValue == null) {
                  throw new JUtilsException(ME + ".UnknownVariable", "Unknown variable " + sub + "");
               }
               value = StringHelper.replaceAll(value, sub, subValue);
            }
            else {
               if (ii > 0) {
                  System.out.println("Property: Replacing '" + key + "=" + jutilsProperties.get(key) + "' to '" + value + "'");
                  jutilsProperties.put(key, value);
               }
               break;
            }
            if (ii > 50) {
               throw new JUtilsException(ME + ".MaxNest", "Maximum nested depth of 10 reached for variable '" + jutilsProperties.get(key) + "'.");
            }
         }
      }
   }

   /**
   * Parse a string to boolean.
   * <p />
   * @param token for example "false"
   * @return true for one of "true", "yes", "1", "ok"<br />
   *         false for "false", "0", "no"
   * @exception if none of the above strings
   */
   static public final boolean toBool(String token) throws JUtilsException {
      if (token == null)
         throw new JUtilsException(ME, "Can't parse <null> to true or false");
      if (token.equalsIgnoreCase("true") || token.equalsIgnoreCase("1") || token.equalsIgnoreCase("ok") || token.equalsIgnoreCase("yes"))
         return true;
      if (token.equalsIgnoreCase("false") || token.equalsIgnoreCase("0") || token.equalsIgnoreCase("no"))
         return false;
      throw new JUtilsException(ME, "Can't parse <" + token + "> to true or false");
   }

   /**
   * Look for properties file.
   * <p />
   * The property file is searched for in the following order:
   * <ol>
   *    <li>Command line parameter '-propertyFile', e.g. "-propertyFile /tmp/xy.properties"</li>
   *    <li>Local directory user.dir</li>
   *    <li>In $PROJECT_HOME, e.g. "/opt/myproject"</li>
   *    <li>In user.home - $HOME</li>
   *    <li>In java.home/lib/ext directory, e.g. "/opt/jdk1.2.2/jre/lib/ext"<br />
   *        You may use this path for Servlets, demons etc.</li>
   *    <li>In java.home directory, e.g. "/opt/jdk1.2.2/jre/lib"<br />
   *        You may use this path for Servlets, demons etc.</li>
   *    <li>Fallback: "\jutils" on DOS-WIN$ or "/usr/local/jutils" on UNIX</li>
   * <ol>
   * <p />
   * @param fileName e.g. "cool.properties"
   * @param args     Only e.g. "-propertyFile /tmp/xy.properties" is evaluated (if given)
   * @return The path to file, e.g. "\jutils\"
   */
   public final String findPath(String fileName, String[] args) {
      if (fileName == null)
         return null;

      File f = null;

      /* Coded in findFile()
      try {
         String argsLocation = Args.getArg(args, "-propertyFile", (String)null);
         if (argsLocation != null) {
            f = new File(argsLocation);
            if (f.exists()) {
               return argsLocation;
            }
            else
               System.out.println("Property: File '" + fileName + "' not given on command line '-propertyFile'");
         }
      } catch(JUtilsException e) {
      }
      */

      f = new File(currentPath, fileName);
      if (f.exists()) {
         return currentPath;
      }
      else
         System.out.println("Property: File '" + fileName + "' is not in current directory " + currentPath);


      if (projectHomePath == null) {
         System.out.println("Property: File '" + fileName + "' is not in PROJECT_HOME, 'java -DPROJECT_HOME=...' is not set ...");
      }
      else {
         if (!projectHomePath.endsWith(separator))
            projectHomePath += separator;
         f = new File(projectHomePath, fileName);
         if (f.exists()) {
            return projectHomePath;
         }
         else
            System.out.println("Property: File '" + fileName + "' is not in directory PROJECT_HOME=" + projectHomePath);
      }

      f = new File(userHome, fileName);
      if (f.exists()) {
         return userHome;
      }
      else
         System.out.println("Property: File '" + fileName + "' is not in user.home directory " + userHome);


      f = new File(javaHomeExt, fileName);
      if (f.exists()) {
         return javaHomeExt;
      }
      else
         System.out.println("Property: File '" + fileName + "' is not in java.home/lib/ext directory " + javaHomeExt);


      f = new File(javaHome, fileName);
      if (f.exists()) {
         return javaHome;
      }
      else
         System.out.println("Property: File '" + fileName + "' is not in java.home directory " + javaHome);
      String guess;
      if (separator.equals("/"))
         guess = "/usr/local/jutils/";
      else
         guess = "\\jutils\\";


      f = new File(guess, fileName);
      if (f.exists()) {
         return guess;
      }
      else
         System.out.println("Property: File '" + fileName + "' is not in directory " + guess);

      return null;
   }

   /**
   * Find properties file.
   * <p />
   * See findPath() for search - logic
   *
   * @param fileName e.g. "cool.properties"
   * @param args Only "-propertyFile /tmp/xy.properties" is evaluated (if given)
   * @return The path to file, e.g. "\jutils\"
      */
   public final String findFile(String fileName, String[] args) {

      try {
         String argsLocation = Args.getArg(args, "-propertyFile", (String)null);
         if (argsLocation != null) {
            File f = new File(argsLocation);
            if (f.exists()) {
               propertyFileName = argsLocation;
               return argsLocation;
            }
            else
               System.out.println("Property: File '" + fileName + "' not given on command line '-propertyFile'");
         }
      } catch(JUtilsException e) {
      }

      if (fileName == null) {
         System.out.println("Property: No property file specified.");
         return null;
      }
      String path = findPath(fileName, args);
      if (path == null) {
         System.err.println("Property: File '" + fileName + "' not found");
         return null;
      }
      String fullName = FileUtil.concatPath(path, fileName);
      // System.out.println("Property: Using file '" + fullName + "'");
      return fullName;
   }

   /**
   * Add key/values, for example from startup command line args to
   * the property variable.
   * <p />
   * Args parameters are stronger and overwrite the property file variables.
   * <p />
   * The arg key must have a leading - or -- (as usual on command line).<br />
   * The leading - are stripped (to match the property variable)
   * args must be a tuple (key / value pairs).
   * <p />
   * Example:
   * <pre>
   *   java org.jutils.Main  isCool true -iorPort 3400
   * </pre>
   * This would overwrite a properties file variable "iorPort" (if there was one)
   * and set the variable isCool to "true"
   */
   public void addArgs2Props(String[] args) throws JUtilsException {
      addArgs2Props(jutilsProperties, args);
      replaceVariables();
   }

   /**
   * See method addArgs2Props(args)
   */
   private void addArgs2Props(Properties props, String[] args) throws JUtilsException {
      if (args == null)
         return;
      for (int ii = 0; ii < args.length; ii++) {
         String arg = args[ii];
         if (arg.startsWith("-")) { // only parameters starting with "-" are recognized
            String key = arg;
            if (arg.equals("--help") || arg.equals("-?") || arg.equals("-h") || arg.equals("-help")) {
               wantsHelp = true;
               continue; // ignore
            }
            if (arg.startsWith("--"))
               key = arg.substring(2); // strip "--"
            else
               if (arg.startsWith("-"))
                  key = arg.substring(1); // strip "-"
            if ((ii + 1) >= args.length) {
               // System.err.println("Property: The property '" + arg + "' has no value");
               throw new JUtilsException(ME + ".MissingValue", "The property '" + arg + "' has no value.");
            }
            String arg2 = args[ii + 1];
            props.put(key, arg2);
            ii++;
            // System.out.println("Property: Setting command line argument " + key + " with value " + arg2);
         }
         else {
            throw new JUtilsException(ME + ".Invalid", "Ignoring unknown argument <" + arg + ">, it should start with '-'.");
         }
      }
   }

   /**
   * Dump all properties to xml representation.
   */
   public String toXml() {
      StringBuffer buf = new StringBuffer();
      buf.append("<Property>\n");
      for (Enumeration e = jutilsProperties.propertyNames(); e.hasMoreElements();) {
         String key = (String) e.nextElement();
         buf.append("   <").append(key).append(">");
         buf.append(StringHelper.replaceAll((String) jutilsProperties.get(key), "<", "&lt;"));
         buf.append("</").append(key).append(">\n");
      }
      buf.append("</Property>\n");
      return buf.toString();
   }

   /**
   * For testing only
   * <p />
   * <pre>
   * java org.jutils.init.Property -Persistence.Driver myDriver -isCool true -xml "<hello>world</hello>"
   * java org.jutils.init.Property -compiler "We use the \${java.compiler} compiler" -double "\${compiler} for \${java.vm.name}"
   * </pre>
   */
   public static void main(String args[]) {
      String ME = "Property";
      try {
         System.out.println("yes=" + Property.toBool("yes"));
         System.out.println("no=" + Property.toBool("no"));
         System.out.println("aaa=" + Property.toBool("aaa"));
      }
      catch (Exception e) {
         System.out.println("OK, aaa=ERROR");
      }
      System.out.println("\n\n");
      try {
         Property props = new Property("xmlBlaster.properties", true, args, true); // initialize

         System.out.println("Persistence=" + props.get("Persistence", false));
         System.out.println("Persistence.Dummy=" + props.get("Persistence.Dummy", false));
         System.out.println("Persistence.Driver=" + props.get("Persistence.Driver", "NONE"));
         System.out.println("Persistence.Dummy=" + props.get("Persistence.Dummy", "NONE"));
         System.out.println("\n\n");
         Properties raw = props.getProperties();
         System.out.println("All properties:\n" + raw);
         System.out.println("All properties as XML:\n" + props.toXml());
      }
      catch (JUtilsException e) {
         //System.err.println(e.toXml(true));
         System.err.println(e.toString());
      }
   }
}
