package org.jutils.jdbc;

import java.sql.*;
/**
 * Provides a connection that can be used to access all Connection functions,
 * plus using a recycling mechanism.
 * To get an instance of this type, call ConnectionPool.getConnection(). To recycle this instance, simply
 * call repool() or close(). Note that PooledConnection.close() will not close the inner connection.
 * ------------------------------------------------------------------------------ 	<br>
 * Name:     	PooledConnection													<br>
 * Project:   	jutils																<br>
 * Copyright:																		<br>
 * Comment:		Provides a pooled connection by wrapping (and implementing) a java.sql.Connection <br>
 * Creation date: (28.06.2000 18:09:41)   											<br>
 * Version:   $Id: PooledConnection.java,v 1.2 2000/07/13 08:43:15 winkler Exp $																<br>
 * ------------------------------------------------------------------------------	<br>
 * @see java.sql.Connection
 * @author: M.Winkler, doubleSlash GmbH
 */
public class PooledConnection implements Connection {
   private Connection c = null;
   private ConnectionPool p = null;
   private boolean locked_ = false;

   /**
    * PooledConnection constructor comment.
    */
   private PooledConnection() {
      super();
   }

   /**
    * Creation date: (28.06.2000 18:23:33)
    * @param: <|>
    * @return:
    * @author: M.Winkler, doubleSlash GmbH
    *
    * @param connection java.sql.Connection
    * @param pool org.jutils.jdbc.ConnectionPool
    */
   protected PooledConnection(Connection connection, ConnectionPool pool) {
      c = connection;
      p = pool;
   }

   /**
    * @see java.sql.Connection
    */
   public void clearWarnings() throws SQLException {
      pCheckLock();
      c.clearWarnings();
   }

   /**
    * When calling this method, instead of closing the wrapped connection, this wrapper is
    * put into the pool. To close a wrapped connection, either call destrroy(),which also
    * will destroy the wrapper, or drop the instance, which will end up in calling the
    * finalize() method, which will close the connection by itself.
    * If the wrapped connection is set to autocommit, then a commit is called to it.
    * @see java.sql.Connection
    */
   public void close() throws SQLException {
      pCheckLock();
      if (c.getAutoCommit())
         c.commit();
      repool();
   }

   /**
    * @see java.sql.Connection
    */
   public void commit() throws SQLException {
      pCheckLock();
      c.commit();
   }

   /**
    * @see java.sql.Connection
    */
   public Statement createStatement() throws SQLException {
      pCheckLock();
      return c.createStatement();
   }

   /**
    * @see java.sql.Connection
    */
   public Statement createStatement(int resultSetType, int resultSetConcurrency) throws SQLException {
      pCheckLock();
      return c.createStatement(resultSetType, resultSetConcurrency);
   }

   /**
    * Closes the inner connection and sets all relevant references to null. All SQLExceptions will be caught
    * and printed, but will not halt execution. CAUTION! Do NOT call manually! this ONLY may be called when:
    * <li><code>PooledConnection.takeInstance()</code> and only if <code>PooledConnection.isOpened ()== false</code>.</li>
    * <li><code>PooledConnection.close()</code> is called and this instance is in the pool.</li>
    * Creation date: (28.06.2000 18:24:49)
    * @author: M.Winkler, doubleSlash GmbH
    *
    */
   protected void destroy() {
      try {
         if ((c != null) && !c.isClosed()) {
            c.close();
         }
      }
      catch (SQLException sqle) {
         System.err.println("Something happened while closing a wrapped connection:" + sqle.toString());
         sqle.printStackTrace(System.err);
         System.err.println("Does not mind. destroying anyway.");
      }
      c = null;
      p = null;
   }

   /**
    * Code to perform when this object is garbage collected. Throws an exception when the inner references are NOT null.
    * In this case, the PooledConnection repools() and throws a RuntimeException indicating that the finalization of this
    * instance should halt.
    */
   protected void finalize() throws Throwable {
      super.finalize();

      // may be finalized?!?
      // this instance only may be finalized, if the pool already is going down,
      // that is if destroy is called before.

      if (p != null) {
         // gotcha!
         locked_ = false;
         p.takeInstance(this);

         // bring finalization to a halt (see API Object.finalize() for that)
         // Juergen: dont panic, its okay... ;)
         throw new RuntimeException("Hey! a lost PooledConnection! Instance has been repooled.");
      }
      else {
         if (c != null) {
            if (!c.isClosed())
               c.close();
            c = null;
         }
      }
   }

   /**
    * @see java.sql.Connection
    */
   public boolean getAutoCommit() throws SQLException {
      pCheckLock();
      return c.getAutoCommit();
   }

   /**
    * @see java.sql.Connection
    */
   public String getCatalog() throws SQLException {
      pCheckLock();
      return c.getCatalog();
   }

   /**
    * Creation date: (04.07.2000 17:53:01)
    * @author: M.Winkler, doubleSlash GmbH
    *
    * @return org.jutils.jdbc.PooledConnection
    */
   public PooledConnection getConnection() {
      pCheckLock();
      return p.getConnection();
   }

   /**
    * @see java.sql.Connection
    */
   public DatabaseMetaData getMetaData() throws SQLException {
      pCheckLock();
      return c.getMetaData();
   }

   /**
    * @see java.sql.Connection.getTransactionIsolation
    */
   public int getTransactionIsolation() throws SQLException {
      pCheckLock();
      return c.getTransactionIsolation();
   }

   /**
    * @see java.sql.Connection
    */
   public java.util.Map getTypeMap() throws SQLException {
      pCheckLock();
      return c.getTypeMap();
   }

   /**
    * @see java.sql.Connection
    */
   public SQLWarning getWarnings() throws SQLException {
      pCheckLock();
      return c.getWarnings();
   }

   /**
    * @see java.sql.Connection
    */
   public boolean isClosed() throws SQLException {
      pCheckLock();
      return c.isClosed();
   }

   /**
    * @see java.sql.Connection
    */
   public boolean isReadOnly() throws SQLException {
      pCheckLock();
      return c.isReadOnly();
   }

   /**
    * When going into a pool, a connection should be locked to prevent further use.
    * If done so, all accesses to public methods will cause a runtime exception.
    * Creation date: (28.06.2000 19:01:25)
    * @param: <|>
    * @return:
    * @author: M.Winkler, doubleSlash GmbH
    *
    */
   protected void lock() {
      pCheckLock();
      locked_ = true;
   }

   /**
    * @see java.sql.Connection
    */
   public String nativeSQL(String sql) throws SQLException {
      pCheckLock();
      return c.nativeSQL(sql);
   }

   /**
    * Creation date: (28.06.2000 19:03:31)
    * @param: <|>
    * @return:
    * @author: M.Winkler, doubleSlash GmbH
    *
    */
   private void pCheckLock() {
      if (locked_)
         throw new RuntimeException("locked!");
   }

   /**
    * @see java.sql.Connection
    */
   public CallableStatement prepareCall(String sql) throws SQLException {
      pCheckLock();
      return c.prepareCall(sql);
   }

   /**
    * @see java.sql.Connection
    */
   public CallableStatement prepareCall(String sql, int resultSetType, int resultSetConcurrency) throws SQLException {
      pCheckLock();
      return c.prepareCall(sql, resultSetType, resultSetConcurrency);
   }

   /**
    * @see java.sql.Connection
    */
   public PreparedStatement prepareStatement(String sql) throws SQLException {
      pCheckLock();
      return prepareStatement(sql);
   }

   /**
    * @see java.sql.Connection
    */
   public PreparedStatement prepareStatement(String sql, int resultSetType, int resultSetConcurrency) throws SQLException {
      pCheckLock();
      return c.prepareStatement(sql, resultSetType, resultSetConcurrency);
   }

   /**
    * Will the instance drop back into the pool.
    * Creation date: (28.06.2000 18:33:57)
    * @param: <|>
    * @return:
    * @author: M.Winkler, doubleSlash GmbH
    *
    */
   public void repool() {
      pCheckLock();
      p.takeInstance(this);
   }

   /**
    * @see java.sql.Connection
    */
   public void rollback() throws SQLException {
      pCheckLock();
      c.rollback();
   }

   /**
    * @see java.sql.Connection
    */
   public void setAutoCommit(boolean autoCommit) throws SQLException {
      pCheckLock();
      c.setAutoCommit(autoCommit);
   }

   /**
    * @see java.sql.Connection
    */
   public void setCatalog(String catalog) throws SQLException {
      pCheckLock();
      c.setCatalog(catalog);
   }

   /**
    * @see java.sql.Connection
    */
   public void setReadOnly(boolean readOnly) throws SQLException {
      pCheckLock();
      c.setReadOnly(readOnly);
   }

   /**
    * @see java.sql.Connection
    */
   public void setTransactionIsolation(int level) throws SQLException {
      pCheckLock();
      c.setTransactionIsolation(level);
   }

   /**
    * @see java.sql.Connection
    */
   public void setTypeMap(java.util.Map map) throws SQLException {
      pCheckLock();
      c.setTypeMap(map);
   }

   /**
    * Removes the lock state (which is not identical with the Java lock pattern!) from this instance.
    * May only be called by the ConnectionPool.
    * If this instance is not locked, a call to unlock() will result in a RuntimeException.
    * Creation date: (28.06.2000 19:04:27)
    * @param: <|>
    * @return:
    * @author: M.Winkler, doubleSlash GmbH
    *
    */
   protected void unlock() {
      if (!locked_)
         throw new RuntimeException("calling unlock() although instance is not locked!");
      locked_ = false;
   }
}
