package org.jutils.log;
/*------------------------------------------------------------------------------
Name:      LogChannel.java
Project:   jutils.org
Copyright: jutils.org, see jutils-LICENSE file
Comment:   The named logging instance which delegates the logging task
           to all registered logging devices in this object.
------------------------------------------------------------------------------*/

import java.text.DateFormat;
import java.text.MessageFormat;
import java.util.Date;
import java.util.Enumeration;
import java.util.Locale;
import java.util.Vector;

import org.jutils.JUtilsException;
import org.jutils.init.Property;


/******************************************************************************
<u>INITIALIZATION BY PARAMETER</u>
<p />

Normaly the configuration and initialization of the LogChannel and all
attached log devices should be done in the main code of the application
which uses this package. This should be done first in the code!<br />
To switch to debug mode or modify logging for testing at startup read the
following information.
<p />
To configure this object and the attached log devices by parameters the
following keywords with the given format must be used:

<pre>
PARAMETERS FOR SETTING DEBUG LEVEL:
   -error true/false         enable/disable error output
   -warn true/false          enable/disable warn output
   -info true/false          enable/disable info output
   -call true/false         enable/disable call output
   -time true/false          enable/disable time output
   -trace true/false         enable/disable trace output
   -dump true/false          enable/disable dump output
   -plain true/false         enable/disable plain output

PARAMETERS FOR DEVICE CONTROL:
   -logConsole true          create new console log device with default setting
   -logFile name             create new file log device with given name + path
   -removeAll true           removes ALL logging devices!

PARAMETERS FOR OUTPUT FORMATTING:
   -LogFormat "{0}:{1}:{2}:{3}"             format: <timestamp>:<levelStr>:<instance>:<text>
   -LogFormat.Date SHORT/MEDIUM/LONG/FULL   MEDIUM is default
   -LogFormat.Time SHORT/MEDIUM/LONG/FULL   MEDIUM is default
   -LogFormat.Language de    both, Language and Country must be set
   -LogFormat.Country de     both, Language and Country must be set
</pre>
******************************************************************************/
public class LogChannel
{
   // global declarations
   public static final int LOG_NOLOG = 0x0;  // No logs at all
   public static final int LOG_ERROR = 0x1;  // Internal error occured
   public static final int LOG_WARN  = 0x2;  // Warning about user actions
   public static final int LOG_INFO  = 0x4;  // Important informational logs only
   public static final int LOG_CALL  = 0x8;  // Trace entry of methods
   public static final int LOG_TIME  = 0x10; // Show elapsed milliseconds
   public static final int LOG_TRACE = 0x20; // Trace application flow
   public static final int LOG_DUMP  = 0x40; // Dump internal state
   public static final int LOG_PLAIN = 0x80; // Marker for plain output


   // member attributes
   private Vector devices = null;
   private Property property = null;

   /***************************************************************************
    * Default log level is  LOG_ERROR | LOG_WARN | LOG_INFO
    **************************************************************************/
   protected int LOGLEVEL = (LOG_ERROR | LOG_WARN | LOG_INFO);

   /***************************************************************************
    * Adjust here your local look and feel
    **************************************************************************/
   private int lookAndFeelDate = java.text.DateFormat.MEDIUM;
   private int lookAndFeelTime = java.text.DateFormat.MEDIUM;
   private Locale country = Locale.getDefault(); // java.util.Locale.US;

   /** format: <timestamp>:<levelStr>:<instance>:<text> */
   private String currentLogFormat = "{0} {1} {2}: {3}";
   private boolean logFormatPropertyRead = false;




   /***************************************************************************
    * The constructor of this object.
    *
    * @param      prop
    *             the property object, which should be used to initialize
    *             this object, or null for default initialization.
    **************************************************************************/
   public LogChannel(Property prop) {
      this.property = prop;
      this.devices = new Vector();
      if (this.property != null) {
         initialize();
      }
   }


   /***************************************************************************
    * Sets the default loglevel LOG_ERROR | LOG_WARN | LOG_INFO
    **************************************************************************/
   public final void setDefaultLogLevel() {
      LOGLEVEL = (LOG_ERROR | LOG_WARN | LOG_INFO);
   }


   /***************************************************************************
    * Initialize this object with given parameters, maybe from commandline
    * at startup.<p />
    * Example:<br />
    * <pre>
    * java myApp.Main -trace true -dump true -call false -dump false
    * </pre>
    *
    * @param      args
    *             the command line arguments.
    **************************************************************************/
   public final void initialize(String[] args) {
      try {
         // initialize with args
         property = new Property(null, false, args, false);
      }
      catch (JUtilsException e) {
         System.err.println(e.toXml(true));
         return;
      }
      initialize(property);
   }


   /***************************************************************************
    * Initialize this object with given parameters.
    *
    * @param      property
    *             the property settings.
    **************************************************************************/
   public final void initialize(Property props) {
      property = props;
      initialize();
   }


   /***************************************************************************
    * Initialize this object.
    **************************************************************************/
   private void initialize() {
      logFormatPropertyRead = true;
      if (property == null) {
         try {
            // initialize dummy
            property = new Property(null, false, null, false);
         }
         catch (JUtilsException e) {
            System.err.println(e.toXml(true));
            return;
         }
      }

      // Given flag '-error false' switches off error messages:
      if (property.propertyExists("error")) {
         if (property.get("error", false))
            addLogLevel("ERROR");
         else
            removeLogLevel("ERROR");
      }
      if (property.propertyExists("warn")) {
         if (property.get("warn", false))
            addLogLevel("WARN");
         else
            removeLogLevel("WARN");
      }
      if (property.propertyExists("info")) {
         if (property.get("info", false))
            addLogLevel("INFO");
         else
            removeLogLevel("INFO");
      }
      if (property.propertyExists("call")) {
         if (property.get("call", false))
            addLogLevel("CALL");
         else
            removeLogLevel("CALL");
      }
      if (property.propertyExists("time")) {
         if (property.get("time", false))
            addLogLevel("TIME");
         else
            removeLogLevel("TIME");
      }
      if (property.propertyExists("trace")) {
         if (property.get("trace", false))
            addLogLevel("TRACE");
         else
            removeLogLevel("TRACE");
      }
      if (property.propertyExists("dump")) {
         if (property.get("dump", false))
            addLogLevel("DUMP");
         else
            removeLogLevel("DUMP");
      }
      if (property.propertyExists("plain")) {
         if (property.get("plain", false))
            addLogLevel("PLAIN");
         else
            removeLogLevel("PLAIN");
      }

      // format: {0}:{1}:{2}:{3}    <timestamp>:<levelStr>:<instance>:<text>
      currentLogFormat = property.get("LogFormat", currentLogFormat);
      String tmp = property.get("LogFormat.Date", "MEDIUM").trim();
      if (tmp.equals("SHORT"))
         lookAndFeelDate = DateFormat.SHORT;
      else
         if (tmp.equals("MEDIUM"))
            lookAndFeelDate = DateFormat.MEDIUM;
         else
            if (tmp.equals("LONG"))
               lookAndFeelDate = DateFormat.LONG;
            else
               if (tmp.equals("FULL"))
                  lookAndFeelDate = DateFormat.FULL;
      tmp = property.get("LogFormat.Time", "MEDIUM").trim();
      if (tmp.equals("SHORT"))
         lookAndFeelTime = DateFormat.SHORT;
      else
         if (tmp.equals("MEDIUM"))
            lookAndFeelTime = DateFormat.MEDIUM;
         else
            if (tmp.equals("LONG"))
               lookAndFeelTime = DateFormat.LONG;
            else
               if (tmp.equals("FULL"))
                  lookAndFeelTime = DateFormat.FULL;
      String la = property.get("LogFormat.Language", (String) null);
      String co = property.get("LogFormat.Country", (String) null);
      if (la != null && co != null)
         country = new Locale(la, co);


      // check for new console devices
      boolean check = property.get("logConsole", false);
      if (check == true) {
         LogDeviceConsole ldc = new LogDeviceConsole(this);
         addLogDevice(ldc);
      }
      // check for new file devices
      String fileName = property.get("logFile", (String) null);
      if (fileName != null) {
         LogDeviceFile ldf = new LogDeviceFile(this, fileName);
         addLogDevice(ldf);
      }
      // remove all devices?
      boolean bRem = property.get("removeAll", false);
      if (bRem == true) {
         devices.clear();
      }
   }


   /***************************************************************************
    * With this method a log device could be added to this object.
    *
    * @param      device
    *             the log device to add.
    **************************************************************************/
   public boolean addLogDevice(LogableDevice device) {
      if (device != null) {
         this.devices.add(device);
         return true;
      }
      return false;
   }


   /***************************************************************************
    * With this method a log device could be removed from this object.
    *
    * @param      device
    *             the log device to remove.
    **************************************************************************/
   public LogableDevice removeLogDevice(LogableDevice device) {
      if (device != null) {
         this.devices.remove(device);
         return device;
      }
      return null;
   }


   /***************************************************************************
    * With this method the format of the logging output can be customized.
    *
    * @param      logFormat
    *             This string must be combined out of the following fields:
    *             <ul>
    *               <li>{0} timestamp</li>
    *               <li>{1} levelStr</li>
    *               <li>{2} instance</li>
    *               <li>{3} text</li>
    *             </ul>
    *             An example would be "{0} # {1} # {2}:  {3}".
    * @param      dateFormat
    *             must be one of the following values: DateFormat.SHORT,
    *             DateFormat.MEDIUM, DateFormat.LONG, DateFormat.FULL
    * @param      timeFormat
    *             must be one of the following values: DateFormat.SHORT,
    *             DateFormat.MEDIUM, DateFormat.LONG, DateFormat.FULL
    * @param      locale
    *             Must be a valid locale or null. If null, the locale is not
    *             modified.
    **************************************************************************/
   public void setLogFormat(String logFormat, int dateFormat, int timeFormat, Locale locale) {
      if (logFormat != null)
         this.currentLogFormat = logFormat;
      if (dateFormat == DateFormat.SHORT || dateFormat == DateFormat.MEDIUM || dateFormat == DateFormat.LONG || dateFormat == DateFormat.FULL)
         this.lookAndFeelDate = dateFormat;
      if (timeFormat == DateFormat.SHORT || timeFormat == DateFormat.MEDIUM || timeFormat == DateFormat.LONG || timeFormat == DateFormat.FULL)
         this.lookAndFeelTime = timeFormat;
      if (locale != null)
         this.country = locale;
   }


   /***************************************************************************
    * Sets the new loglevel to the given value.
    *
    * @param      level
    *             the new loglevel value.
    **************************************************************************/
   public void setLogLevel(int level) {
      LOGLEVEL = level;
   }


   /***************************************************************************
    * Gets the actual loglevel.
    *
    * @return     a binary OR-ed value of the basic values
    *             LogChannel.LOG_ERROR, LogChannel.LOG_WARN,
    *             LogChannel.LOG_INFO, ...
    **************************************************************************/
   public int getLogLevel() {
      return LOGLEVEL;
   }


   /***************************************************************************
    * Adds the loglevel
    *
    * @param      for example "DUMP"
    **************************************************************************/
   public void addLogLevel(String logLevel) {
      int level = logLevelToBit(logLevel);
      LOGLEVEL = (LOGLEVEL | level);
   }


   /***************************************************************************
    * Removes the loglevel
    *
    * @param      for example "TRACE"
    **************************************************************************/
   public void removeLogLevel(String logLevel) {
      int level = logLevelToBit(logLevel);
      LOGLEVEL = (LOGLEVEL & ~level);
   }


   /***************************************************************************
    * Converts the String logLevel (e.g. "ERROR") to the bit setting
    * LogChannel.LOG_ERROR.
    *
    * @param      for example "TRACE"
    * @return     bit setting LogChannel.LOG_TRACE
    **************************************************************************/
   private static final int logLevelToBit(String logLevel) {
      if (logLevel == null)
         return LOG_NOLOG;
      if (logLevel.equals("ERROR"))
         return LOG_ERROR;
      else
         if (logLevel.equals("WARN"))
            return LOG_WARN;
         else
            if (logLevel.equals("INFO"))
               return LOG_INFO;
            else
               if (logLevel.equals("CALL"))
                  return LOG_CALL;
               else
                  if (logLevel.equals("TIME"))
                     return LOG_TIME;
                  else
                     if (logLevel.equals("TRACE"))
                        return LOG_TRACE;
                     else
                        if (logLevel.equals("DUMP"))
                           return LOG_DUMP;
      System.err.println("Unknown logging level '" + logLevel + "'");
      return LOG_NOLOG;
   }


   /***************************************************************************
    * This method creates a well formatted string out of all the given data
    * depending on the settings which are stored in this object by the
    * user / application.<p />
    * Because only this object knows this settings and only the log devices
    * know the specific output strings for the different debug levels, the
    * log devices normally use this method to format the logging data.
    *
    * @param      levelString
    *             the formatted string representation for the given debug
    *             level (specified in the log devices).
    * @param      level
    *             the value of the debug level, allowed values are
    *             LogChannel.LOG_ERROR, LogChannel.LOG_WARN,
    *             LogChannel.LOG_INFO, ...
    * @param      source
    *             the name or identifier of the initiator of this logging
    * @param      text
    *             the data which should be logged
    * @return     a well formatted string which could be logged.
    **************************************************************************/
   public String formatLogData(String levelStr, int level, String source, String text) {
      String logFormat;
      if ((level & LOG_PLAIN) != 0) {
         logFormat = "{3}";
      }
      else {
         logFormat = currentLogFormat;
      }
      Date currentDate = new Date();
      DateFormat df = DateFormat.getDateTimeInstance(lookAndFeelDate, lookAndFeelTime, country);
      String timeStr = df.format(currentDate);
      Object[] arguments = {timeStr, levelStr, source, text};
      String logEntry = MessageFormat.format(logFormat, arguments);
      return logEntry;
   }


   /***************************************************************************
    * This method delegates all incoming logging request to all registered
    * log devices. If no log devices are registered nothing happens.
    *
    * @param      level
    *             the value of the debug level, allowed values are
    *             LogChannel.LOG_ERROR, LogChannel.LOG_WARN,
    *             LogChannel.LOG_INFO, ...
    * @param      source
    *             the name or identifier of the initiator of this logging
    * @param      text
    *             the data which should be logged
    **************************************************************************/
   public void log(int level, String source, String text) {
      for (Enumeration e = devices.elements(); e.hasMoreElements();) {
         LogableDevice tmp = (LogableDevice) e.nextElement();
         if (tmp != null) {
            tmp.log(level, source, text);
         }
      }
   }


   /***************************************************************************
    * Use this for reporting internal errors. Depending on the
    * system / application / situation the execution must be stopped.
    * This stop must be done in the application.
    *
    * @param      source
    *             the name or identifier of the initiator of this logging
    * @param      text
    *             the string to log
    **************************************************************************/
   public void error(String source, String text) {
      if ((LOGLEVEL & LOG_ERROR) != 0) {
         log(LogChannel.LOG_ERROR, source, text);
      }
   }


   /***************************************************************************
    * Use this for logging output which is important, for example a
    * 'login denied' event.
    *
    * @param      source
    *             the name or identifier of the initiator of this logging
    * @param      text
    *             the string to log
    **************************************************************************/
   public void warn(String source, String text) {
      if ((LOGLEVEL & LOG_WARN) != 0) {
         log(LogChannel.LOG_WARN, source, text);
      }
   }


   /***************************************************************************
    * Use this for normal logging output.
    *
    * @param      source
    *             the name or identifier of the initiator of this logging
    * @param      text
    *             the string to log
    **************************************************************************/
   public void info(String source, String text) {
      if ((LOGLEVEL & LOG_INFO) != 0) {
         log(LogChannel.LOG_INFO, source, text);
      }
   }


   /***************************************************************************
    * Tracing when entering methods.
    *
    * @param      source
    *             the name or identifier of the initiator of this logging
    * @param      text
    *             the string to log
    **************************************************************************/
   public void call(String source, String text) {
      if ((LOGLEVEL & LOG_CALL) != 0) {
         log(LogChannel.LOG_CALL, source, text);
      }
   }


   /***************************************************************************
    * Output of performant measurements (elapsed milliseconds).
    *
    * @param      source
    *             the name or identifier of the initiator of this logging
    * @param      text
    *             the string to log
    **************************************************************************/
   public void time(String source, String text) {
      if ((LOGLEVEL & LOG_TIME) != 0) {
         log(LogChannel.LOG_TIME, source, text);
      }
   }


   /***************************************************************************
    * Trace the execution.
    *
    * @param      source
    *             the name or identifier of the initiator of this logging
    * @param      text
    *             the string to log
    **************************************************************************/
   public void trace(String source, String text) {
      if ((LOGLEVEL & LOG_TRACE) != 0) {
         log(LogChannel.LOG_TRACE, source, text);
      }
   }


   /***************************************************************************
    * Log text as given, no extra data attached.
    *
    * @param      source
    *             the name or identifier of the initiator of this logging
    * @param      text
    *             the string to log
    **************************************************************************/
   public void plain(String source, String text) {
      if ((LOGLEVEL & LOG_PLAIN) != 0) {
         log(LogChannel.LOG_PLAIN, source, text);
      }
   }


   /***************************************************************************
    * main
    *
    * There are implemented many test cases.
    * You can run all existing test after each modification at this class
    * by just typing:
    *    java org.jutils.log.LogChannel
    *
    * @ param     args
    *             all given parameter
    **************************************************************************/
   public static void main(String[] args) {
      LogChannel log = new LogChannel(null);

      // Test with 0 devices
      System.out.println("TEST 1:");
      System.out.println("Test if NO logging device does exist.");
      log.error("Source1", "Text1");
      log.warn("Source2", "Text2");
      log.info("Source3", "Text3");
      log.call("Source4", "Text4");
      log.time("Source5", "Text5");
      log.trace("Source6", "Text6");
      log.plain("Source7", "Text7");
      System.out.println("END TEST 1\n");

      // Test with 1 console device
      System.out.println("TEST 2:");
      System.out.println("Test with 1 console logging device with standard LOGLEVEL");
      LogDeviceConsole logConsole1 = new LogDeviceConsole(log);
      log.addLogDevice(logConsole1);
      log.error("Source1", "Text1");
      log.warn("Source2", "Text2");
      log.info("Source3", "Text3");
      log.call("Source4", "Text4");
      log.time("Source5", "Text5");
      log.trace("Source6", "Text6");
      log.plain("Source7", "Text7");
      System.out.println("END TEST 2\n");

      // Test with 1 console device
      System.out.println("TEST 3:");
      System.out.println("Test with 1 console logging device with all LOGLEVELS activated");
      log.setLogLevel(LOG_ERROR | LOG_WARN | LOG_INFO | LOG_CALL | LOG_TIME | LOG_TRACE | LOG_DUMP | LOG_PLAIN);
      log.error("Source1", "Text1");
      log.warn("Source2", "Text2");
      log.info("Source3", "Text3");
      log.call("Source4", "Text4");
      log.time("Source5", "Text5");
      log.trace("Source6", "Text6");
      log.plain("Source7", "Text7");
      System.out.println("END TEST 3\n");
      log.initialize(args);
      System.out.println("###\nFrom now on the user defined debug levels are used!\n###");

      // Test with 2 console devices
      System.out.println("TEST 4:");
      System.out.println("Test with 2 console logging devices, one without color");
      LogDeviceConsole logConsole2 = new LogDeviceConsole(log);
      logConsole2.setColorMode(false);
      log.addLogDevice(logConsole2);
      log.error("Source1", "Text1");
      log.warn("Source2", "Text2");
      log.info("Source3", "Text3");
      log.call("Source4", "Text4");
      log.time("Source5", "Text5");
      log.trace("Source6", "Text6");
      log.plain("Source7", "Text7");
      log.removeLogDevice(logConsole2);
      System.out.println("END TEST 4\n");

      // Test with 1 console device and 1 file device
      System.out.println("TEST 5:");
      System.out.println("Test with 1 console device and 1 file device");
      LogDeviceFile logFile1 = new LogDeviceFile(log, "./test.log");
      log.addLogDevice(logFile1);
      log.error("Source1", "Text1");
      log.warn("Source2", "Text2");
      log.info("Source3", "Text3");
      log.call("Source4", "Text4");
      log.time("Source5", "Text5");
      log.trace("Source6", "Text6");
      log.plain("Source7", "Text7");
      log.removeLogDevice(logConsole1);
      System.out.println("END TEST 5\n");

/*
      // Test with 1 file device and more than 50000 lines entry
      System.out.println("TEST 6:");
      System.out.println("Now generating 60000 lines of debug output with 'log.info' in a file, please stand by ...");
      for (int i = 0; i < 60000; i++) {
         if (((i+1) % 10000) == 0)
            System.err.println("   " + (i+1) + " lines generated...");
         log.info("Source"+ i, "Text" + i);
      }
      System.out.println("END TEST 6 -> look for test.log* files in actual dir!\n");
*/
   }
}
