package org.jutils.log;
/*------------------------------------------------------------------------------
Name:      LogManager.java
Project:   jutils.org
Copyright: jutils.org, see jutils-LICENSE file
Comment:   This logging manager object handles multiple 'named' logging
           instances (channels) which could be generated by one program for
           different logging purposes or which could be generated by many
           programs in the same VM. Each name must be unique.
------------------------------------------------------------------------------*/

import java.util.Hashtable;

import org.jutils.JUtilsException;


/******************************************************************************
<U>INITIALIZATION BY PARAMETER</U>

To configure the connected LogChannel objects in this object all known
parameters which are defined by LogChannel (see LogChannel class description)
can be used but must have the channel name + "." as prefix, e.g. the
logging channel is named 'jutils' the parameter for enabling the debug
level 'info' of this channel would be '-jutils.info true'.
******************************************************************************/
public class LogManager
{
   private final static String DEFAULT_CHANNEL = "default";
   private static Hashtable channels = new Hashtable();
   // static pre initialization
   static {
      initialize();
   };




   /***************************************************************************
    *
    **************************************************************************/
   public static void initialize() {
      LogChannel tmp = new LogChannel(null);
      tmp.addLogDevice(new LogDeviceConsole(tmp));
      channels.put(DEFAULT_CHANNEL, tmp);
   }


   /***************************************************************************
    *
    **************************************************************************/
   public static boolean addLogChannel(String channelName, LogChannel log) {
      if ((channelName != null) && (!channelName.equals("")) && (log != null)) {
         channels.put(channelName, log);
         return true;
      }
      return false;
   }


   /***************************************************************************
    *
    **************************************************************************/
   public static LogChannel getLogChannel(String channelName) {
      LogChannel log = (LogChannel) channels.get(channelName);
      return log;
   }


   /***************************************************************************
    *
    **************************************************************************/
   public static void log(String channelName, int level, String source, String text) {
      if ((channelName == null) || (channelName.equals(""))) {
         return;
      }
      LogChannel channel = (LogChannel) channels.get(channelName);
      if (channel != null) {
         channel.log(level, source, text);
      }
      else {
         if (!channelName.equals(DEFAULT_CHANNEL)) {
            log(DEFAULT_CHANNEL, LogChannel.LOG_ERROR, source, "logchannel '" + channelName + "' not found!");
         }
      }
   }


   /***************************************************************************
    *
    **************************************************************************/
   public static void log(String text) {
      log(DEFAULT_CHANNEL, LogChannel.LOG_PLAIN, "", text);
   }


   /***************************************************************************
    * Use this for reporting internal errors. Depending on the
    * system / application / situation the execution must be stopped.
    * This stop must be done in the application.
    *
    * @param      channelName
    *             the selection, to which channel this log should be sent
    * @param      source
    *             the identifier of the sender
    * @param      text
    *             the string to log
    **************************************************************************/
   public static void error(String channelName, String source, String text) {
      log(channelName, LogChannel.LOG_ERROR, source, text);
   }


   /***************************************************************************
    * Use this for logging output which is important, for example a
    * 'login denied' event.
    *
    * @param      channelName
    *             the selection, to which channel this log should be sent
    * @param      source
    *             the identifier of the sender
    * @param      text
    *             the string to log
    **************************************************************************/
   public static void warn(String channelName, String source, String text) {
      log(channelName, LogChannel.LOG_WARN, source, text);
   }


   /***************************************************************************
    * Use this for normal logging output.
    *
    * @param      channelName
    *             the selection, to which channel this log should be sent
    * @param      source
    *             the identifier of the sender
    * @param      text
    *             the string to log
    **************************************************************************/
   public static void info(String channelName, String source, String text) {
      log(channelName, LogChannel.LOG_INFO, source, text);
   }


   /***************************************************************************
    * Tracing when entering methods.
    *
    * @param      channelName
    *             the selection, to which channel this log should be sent
    * @param      source
    *             the identifier of the sender
    * @param      text
    *             the string to log
    **************************************************************************/
   public static void call(String channelName, String source, String text) {
      log(channelName, LogChannel.LOG_CALL, source, text);
   }


   /***************************************************************************
    * Output of performant measurements (elapsed milliseconds).
    *
    * @param      channelName
    *             the selection, to which channel this log should be sent
    * @param      source
    *             the identifier of the sender
    * @param      text
    *             the string to log
    **************************************************************************/
   public static void time(String channelName, String source, String text) {
      log(channelName, LogChannel.LOG_TIME, source, text);
   }


   /***************************************************************************
    * Trace the execution.
    *
    * @param      channelName
    *             the selection, to which channel this log should be sent
    * @param      source
    *             the identifier of the sender
    * @param      text
    *             the string to log
    **************************************************************************/
   public static void trace(String channelName, String source, String text) {
      log(channelName, LogChannel.LOG_TRACE, source, text);
   }


   /***************************************************************************
    * Log text as given, no extra data attached.
    *
    * @param      channelName
    *             the selection, to which channel this log should be sent
    * @param      source
    *             the identifier of the sender
    * @param      text
    *             the string to log
    **************************************************************************/
   public static void plain(String channelName, String source, String text) {
      log(channelName, LogChannel.LOG_PLAIN, source, text);
   }


   /***************************************************************************
    *
    **************************************************************************/
   public static void main(String[] args) {
      // Test 1
      System.out.println("\n### BEGIN TEST 1 ###");
      LogManager.log("Test1");
      LogManager.log("default", LogChannel.LOG_ERROR, "LogManager", "Test1");
      System.out.println("### END TEST 1 ###\n");


      // Test 2
      System.out.println("\n### BEGIN TEST 2 ###");
      LogChannel log1 = new LogChannel(null);
      LogDeviceConsole console1 = new LogDeviceConsole(log1);
      console1.setColorMode(true);
      log1.addLogDevice(console1);
      LogManager.addLogChannel("log1", log1);
      LogChannel log2 = new LogChannel(null);
      LogDeviceConsole console2 = new LogDeviceConsole(log2);
      console2.setColorMode(false);
      log2.addLogDevice(console2);
      LogManager.addLogChannel("log2", log2);
      LogManager.log("log1", LogChannel.LOG_INFO, "LogManager.main()", "Output1");
      LogManager.log("log2", LogChannel.LOG_ERROR, "LogManager.main()", "Output2");
      System.out.println("### END TEST 2 ###\n");


      // Test 3
      System.out.println("\n### BEGIN TEST 3 ###");
      LogManager.error("log1", "LogManager.main()", "3_1");
      LogManager.warn("log1", "LogManager.main()", "3_2");
      LogManager.info("log1", "LogManager.main()", "3_3");
      LogManager.call("log1", "LogManager.main()", "3_4");
      LogManager.time("log1", "LogManager.main()", "3_5");
      LogManager.trace("log1", "LogManager.main()", "3_6");
      LogManager.plain("log1", "LogManager.main()", "3_7");
      System.out.println("### END TEST 3 ###\n");
   }
}
